import { Store, GetterTree, MutationTree, ActionTree, DispatchOptions, CommitOptions } from 'vuex';
declare type MergedFunctionProcessor<T extends () => any, O> = Parameters<T>[1] extends undefined ? (options?: O) => ReturnType<T> : (payload: Parameters<T>[1], options?: O) => ReturnType<T>;
declare type GettersTransformer<T extends Record<string, any>> = Readonly<{
    [P in keyof T]: ReturnType<T[P]>;
}>;
declare type MutationsTransformer<T extends Record<string, any>> = {
    [P in keyof T]: MergedFunctionProcessor<T[P], CommitOptions>;
};
declare type ActionTransformer<T extends Record<string, any>> = {
    [P in keyof T]: MergedFunctionProcessor<T[P], DispatchOptions>;
};
declare type ModuleTransformer<T> = T extends NuxtModules ? {
    [P in keyof T]: MergedStoreType<T[P]>;
} : {};
interface BlankStore {
    getters: {};
    mutations: {};
    actions: {};
    modules: {};
}
interface NuxtStore {
    state: () => unknown;
    getters: Record<string, any>;
    mutations: Record<string, any>;
    actions: Record<string, any>;
    modules: NuxtModules;
}
declare type NuxtModules = Record<string, NuxtStore>;
interface NuxtStoreInput<T extends () => any, G, M, A, S extends {
    [key: string]: NuxtStore;
}> {
    state: T;
    getters?: G;
    mutations?: M;
    actions?: A;
    modules?: S;
}
declare type MergedStoreType<T extends NuxtStore> = ReturnType<T['state']> & GettersTransformer<T['getters']> & MutationsTransformer<T['mutations']> & ActionTransformer<T['actions']> & ModuleTransformer<T['modules']>;
declare type StoreParameter<T extends () => any> = Parameters<T>[1] extends undefined ? never : Parameters<T>[1];
interface Dispatch<T extends Record<string, () => any>> {
    <P extends keyof T>(action: P, payload: StoreParameter<T[P]>, options?: DispatchOptions): ReturnType<T[P]>;
    <P extends keyof T>(action: StoreParameter<T[P]> extends never ? P : never, options?: DispatchOptions): ReturnType<T[P]>;
}
interface Commit<T extends Record<string, () => any>> {
    <P extends keyof T>(mutation: P, payload: StoreParameter<T[P]>, options?: DispatchOptions): ReturnType<T[P]>;
    <P extends keyof T>(mutation: StoreParameter<T[P]> extends never ? P : never, options?: CommitOptions): ReturnType<T[P]>;
}
export declare type ActionContext<T extends Required<NuxtStore>> = {
    state: ReturnType<T['state']>;
    getters: {
        [P in keyof T['getters']]: ReturnType<T['getters'][P]>;
    };
    commit: Commit<T['mutations']>;
    dispatch: Dispatch<T['actions']>;
    rootState: any;
    rootGetters: any;
};
export declare const getStoreType: <T extends () => any, G, M, A, S extends Record<string, NuxtStore>>(store: NuxtStoreInput<T, G, M, A, S>) => {
    actionContext: ActionContext<NuxtStoreInput<T, G, M, A, S> & BlankStore>;
    storeInstance: ActionContext<NuxtStoreInput<T, G, M, A, S> & BlankStore> & Pick<Store<ReturnType<T>>, "app" | "replaceState" | "subscribe" | "subscribeAction" | "watch" | "registerModule" | "unregisterModule" | "hotUpdate" | "$router">;
};
export declare const getAccessorType: <T extends () => any, G extends GetterTree<ReturnType<T>, ReturnType<T>>, M extends MutationTree<ReturnType<T>>, A extends ActionTree<ReturnType<T>, ReturnType<T>>, S extends Record<string, NuxtStore>>(store: NuxtStoreInput<T, G, M, A, S>) => MergedStoreType<NuxtStoreInput<T, G, M, A, S> & BlankStore>;
export declare const useAccessor: <T extends () => any, G extends GetterTree<ReturnType<T>, ReturnType<T>>, M extends MutationTree<ReturnType<T>>, A extends ActionTree<ReturnType<T>, ReturnType<T>>, S extends Record<string, NuxtStore>>(store: Store<ReturnType<T>>, input: Required<NuxtStoreInput<T, G, M, A, S>>) => MergedStoreType<NuxtStoreInput<T, G, M, A, S> & BlankStore>;
export declare const getAccessorFromStore: (pattern: any) => (store: Store<any>) => any;
export {};
